;
; You need NSIS (http://nsis.sourceforge.net/download/) to compile this.
;

!define AmmoName	"WinDVD"
!define AmmoShort	"ZWINDVD"
!define AmmoDLL		"ZWINDVD.dll"
!define AmmoSPath	"Release"
!define AmmoPackage	"..\wdp2xx.exe"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetCompressor "LZMA"

!define ATISATMRC   "Software\ATI Technologies\Multimedia\Remote Control"
!define ATIPath     $R8
!define ATIHWND     $R9

!include "MUI.nsh"


Name "${AmmoName} Plugin"
OutFile "${AmmoPackage}"
InstallDir "$PROGRAMFILES\Remote Wonder Plugins"
InstallDirRegKey HKLM "SOFTWARE\ATI Technologies\Multimedia\Remote Wonder" "PlugInsPath"

!insertmacro MUI_PAGE_LICENSE "license.txt"
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_WELCOME
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH

!insertmacro MUI_LANGUAGE "English"



Function LoadATIRW
	IntCmp ${ATIHWND} 0 QUIT
	Sleep 3000
	FindWindow ${ATIHWND} "AfxFrameOrView70" "ATIRemoteWonder" 0 0
	IntCmp ${ATIHWND} 0 +1 QUIT
		Exec ${ATIPath}
		StrCpy ${ATIHWND} 0
QUIT:   
FunctionEnd

Function .onInit

	StrCpy $INSTDIR "$INSTDIR\${AmmoShort}"
	
	StrCpy ${ATIHWND} 0
	
	;
	; can we find ATIRW.EXE?
	;
	ReadRegStr $R0 HKLM "SOFTWARE\ATI Technologies\Multimedia\Remote Wonder" "PlugInsPath"
	StrCpy ${ATIPath} "$R0\..\ATIRW.EXE"
	IfFileExists ${ATIPath} +1 NOATI
	
	;
	; can we find the "plug-ins" registry?
	;
	StrCpy $R4 0
AGAIN:   
	EnumRegKey $R5 HKCU "${ATISATMRC}" $R4
	StrCmp $R5 "" NOATI
	StrCmp $R5 "Plug-Ins" OK
	IntOp $R4 $R4 + 1
	goto AGAIN

NOATI:
	StrCpy ${ATIPath} ""

OK:
FunctionEnd

Function .onInstFailed
   call LoadATIRW
FunctionEnd
  
Section "${AmmoName} Plugin" SecCopy
	;
	; find ATIRW, close it temporarily
	;
	StrCmp ${ATIPath} "" WARN
	FindWindow ${ATIHWND} "AfxFrameOrView70" "ATIRemoteWonder" 0 0
	IntCmp ${ATIHWND} 0 WARN
	SendMessage ${ATIHWND} ${WM_CLOSE} 0 0
	Sleep 3000
	goto FILECOPY
     
WARN:   
	MessageBox MB_OK|MB_ICONINFORMATION \
	 "If you are updating an existing plugin, please close the ATI Remote Wonder software temporarily."

FILECOPY:   
	SetOutPath "$INSTDIR"
	File "${AmmoSPath}\${AmmoDLL}"
;	File /oname=${AmmoShort}.txt readme.txt
	IfFileExists "$INSTDIR\${AmmoShort}.ini" +2
	File /oname=${AmmoShort}.ini default.ini
	
	WriteUninstaller "$INSTDIR\remove${AmmoShort}.exe"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${AmmoName}Ammo" "DisplayName" "${AmmoName} Plugin for ATI Remote Wonder (remove)"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${AmmoName}Ammo" "UninstallString" "$INSTDIR\remove${AmmoShort}.exe"
	
	
	;
	; 
	;
	StrCmp ${ATIPath} "" MANUAL
   
	;
	; delete existing registry entry if needed...
	;
	StrCpy $R4 0
AGAIN:   
	EnumRegKey $R5 HKCU "${ATISATMRC}\Plug-Ins" $R4
	StrCmp $R5 "" NOMATCH
	StrLen $R3 ${AmmoName}
	IntOp $R3 $R3 + 8
	StrCpy $R6 $R5 $R3
	StrCmp $R6 "${AmmoName} Plugin " FOUND
	IntOp $R4 $R4 + 1
	goto AGAIN
FOUND:
	DeleteRegKey HKCU "${ATISATMRC}\Plug-Ins\$R5"
NOMATCH:   
	
	;
	; create a new registry entry for this plugin
	;
	GetDLLVersionLocal "${AmmoSPath}\${AmmoDLL}" $R4 $R5
	IntOp $R5 $R4 & 0xFFFF
	IntFmt $R5 "%02u" $R5
	IntOp $R4 $R4 / 0xFFFF
	WriteRegStr HKCU "${ATISATMRC}\Plug-Ins\${AmmoName} Plugin $R4.$R5" "Filename" "$INSTDIR\${AmmoDLL}"
	WriteRegDWORD HKCU "${ATISATMRC}\Plug-Ins\${AmmoName} Plugin $R4.$R5" "Enabled" 1
	
	;
	; wake her up...
	;
	call LoadATIRW
   
	goto QUIT


MANUAL:
	MessageBox MB_OK|MB_ICONINFORMATION \
	 "The plugin was installed, but not loaded into ATI Remote Wonder. To finish the installation, open ATI Remote Wonder and import $INSTDIR\${AmmoDLL}."
	
QUIT:
SectionEnd



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Section "Uninstall"
	MessageBox MB_OK|MB_ICONINFORMATION \
	 "NOTE: Please close the ATI Remote Wonder software temporarily so files that are in use are removed properly."
	
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${AmmoName}Ammo"
	Delete "$INSTDIR\${AmmoDLL}"
	Delete "$INSTDIR\${AmmoShort}.txt"
	Delete "$INSTDIR\${AmmoShort}.ini"
	Delete "$INSTDIR\remove${AmmoShort}.exe"
	RmDir "$INSTDIR"

	;
	; remove from ATI's registry
	;
	StrCpy $R4 0
AGAIN:   
	EnumRegKey $R5 HKCU "${ATISATMRC}\Plug-Ins" $R4
	StrCmp $R5 "" DONE
	StrLen $R3 ${AmmoName}
	IntOp $R3 $R3 + 8
	StrCpy $R6 $R5 $R3
	StrCmp $R6 "${AmmoName} Plugin " FOUND
	IntOp $R4 $R4 + 1
	goto AGAIN
FOUND:
   DeleteRegKey HKCU "${ATISATMRC}\Plug-Ins\$R5"

DONE:
SectionEnd


