/*    
    WinDVD Plugin for ATI Remote Wonder
    Copyright 2002-2005 Jonathan Zarate
	http://www.polarcloud.com
	info@polarcloud.com

    This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

    This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
	for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "stdafx.h"
#include "config.h"
#include "global.h"
#include "actioncodes.h"
#include "mixer.h"
extern "C" {
#include "ammo.h"
}

HWND hVideo;
HWND hVideoSkin;
HWND hLastVideo = NULL;
HWND hPlayer;
HWND hPlayerSkin;
HWND hLang;
HWND hNav;
TCHAR *WinTitle = NULL;
BYTE AudioChannel = 2; // default: Both

// Tested:
//  7.0 B27.066
//		-
//	6.0 B06.056 ENG
//		- OK
//	5.3 B05.037 ENG
//		- OK


void GetTitle(int v)
{
	HKEY hKey;
	DWORD n;
	TCHAR s[128];

	_stprintf(s, _T("Software\\InterVideo\\DVD%d"), v);
	if (RegOpenKeyEx(HKEY_CURRENT_USER, s, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS) {
		n = sizeof(s);
		if (RegQueryValueEx(hKey, _T("AppCaption"), NULL, NULL, (LPBYTE)s, &n) == ERROR_SUCCESS) {
			WinTitle = _tcsdup(s);
		}
		RegCloseKey(hKey);
	}
}

BOOL APIENTRY DllMain(HINSTANCE hDLLInst, DWORD  dwReason, LPVOID) 
{
	if (dwReason != DLL_PROCESS_ATTACH) return TRUE;

	for (int i = 10; i >= 5; --i) {
		GetTitle(i);
		if (WinTitle) {
			hInst = hDLLInst;
			LoadConfig();
			return TRUE;
		}
	}

	MessageBox(NULL, _T("Unable to find AppCaption"), _T("WINDVD Plugin"), MB_OK|MB_ICONSTOP);
	return FALSE;
}


// ugly... but SetForegroundWindow() will not work across processes without these
void ForceForeground(HWND hWnd) 
{
	DWORD a;
	DWORD it;

	if (IsIconic(hWnd)) ShowWindow(hWnd, SW_RESTORE);
	if (GetForegroundWindow() == hWnd) return;
	a = GetWindowThreadProcessId(GetForegroundWindow(), NULL);
	it = GetWindowThreadProcessId(hWnd, NULL);
	AttachThreadInput(a, it, TRUE);
	BringWindowToTop(hWnd);
	SetForegroundWindow(hWnd);
	AttachThreadInput(a, it, FALSE);
	SetForegroundWindow(hWnd);	// ?
}

void KeyDown(DWORD Code)
{
	if (Code & ACTF_KEY_CTRL) keybd_event(VK_CONTROL, 0, 0, 0);
//	if (Code & ACTF_KEY_ALT) keybd_event(VK_MENU, 0, 0, 0);
//	if (Code & ACTF_KEY_SHIFT) keybd_event(VK_LSHIFT, 0, 0, 0);
	keybd_event((BYTE)(Code & ACTF_KEY_VALUE_MASK), 0, 0, 0);
	keybd_event((BYTE)(Code & ACTF_KEY_VALUE_MASK), 0, KEYEVENTF_KEYUP, 0);
//	if (Code & ACTF_KEY_SHIFT) keybd_event(VK_LSHIFT, 0, KEYEVENTF_KEYUP, 0);
//	if (Code & ACTF_KEY_ALT) keybd_event(VK_MENU, 0, KEYEVENTF_KEYUP, 0);
	if (Code & ACTF_KEY_CTRL) keybd_event(VK_CONTROL, 0, KEYEVENTF_KEYUP, 0);
}

DWORD WhatKeysDoYouWant(void) 
{
	return (CUSTOM_RAW|MOUSE_GROUP|CHANNEL_GROUP|VOLUME_GROUP|NUMBER_GROUP|
		CURSOR_GROUP|PLAY_GROUP|MENU|SETUP|ENTER|RECORD|STOPWATCH|RESIZE|WEB_LAUNCH);
}

char *EnumerateProgrammableFunction(WORD wIndex) 
{
	return NULL;
}

void Configure(HWND hWnd) 
{
	ConfigDialog(hWnd);
}

bool CheckWindows(void)
{

	// find the video window and the player window and their "skin" (?) layers

	if ((hVideo = FindWindow(_T("WinDVDClass"), WinTitle)) == NULL) return false;
	if (hVideo != hLastVideo) {
		if ((hVideoSkin = FindWindowEx(hVideo, NULL, _T("AfxOleControl42"), _T(""))) == NULL) return false;
		// player docked
		if ((hPlayer = FindWindowEx(hVideo, NULL, _T("WinDVDClass"), _T("Player"))) == NULL) {
			// player undocked
			hPlayer = NULL;
			while ((hPlayer = FindWindowEx(NULL, hPlayer, _T("WinDVDClass"), _T("Player"))) != NULL) {
				if (GetWindowLong(hPlayer, GWL_HWNDPARENT) == (LONG)hVideo) break;
			}
			if (hPlayer == NULL) return false;
		}
		if ((hPlayerSkin = FindWindowEx(hPlayer, NULL, _T("AfxOleControl42"), _T(""))) == NULL) return false;

		hLang = 0;
		hNav = 0;
		hLastVideo = hVideo;
	}

	return true;
}


BOOL HandleKey (BOOL bCustom, WORD wKeyEvent, WORD wState) 
{
	DWORD ActionCode;

	// things that should not be here
	if (bCustom) return FALSE;
	if (wKeyEvent >= MAX_WONDER_KEYS) return FALSE;

	// keycode -> actioncode
	ActionCode = ActionMap[wKeyEvent];// & ~ACTF_ALTCODE;

	if (ActionCode == ACT_NONE) return FALSE;
	if (ActionCode == ACT_IGNORE) return TRUE;

	if (!CheckWindows()) {
		MessageBox(NULL, _T("Unable to find a WinDVD window."), _T("WINDVD Plugin"), MB_OK|MB_ICONSTOP);
		return TRUE;
	}

	if (((wState == RMCTRL_KEY_ON) && ((ActionCode & ACTF_NO_DOWN) == 0)) ||
		((wState == RMCTRL_KEY_REPEAT) && (ActionCode & ACTF_ON_REPEAT)) ||
		((wState == RMCTRL_KEY_OFF) && (ActionCode & ACTF_ON_UP))) {

//		MessageBeep(-1);

		switch (ActionCode & ACTF_F_MASK) {
		case ACTF_F_SPECIAL:
			switch (ActionCode) {
			case ACT_MIXVOLUP:
			case ACT_MIXVOLDN:
			case ACT_MIXVOLMUTE:
				MixerEvent(ActionCode, wState);
				break;
			case ACT_AUDIOCHAN:
				//	38453	None (??)
				//	38454	Left
				//	38455	Right
				//	38456	Both
				AudioChannel = (AudioChannel + 1) % 3;
				PostMessage(hPlayerSkin, WM_COMMAND, 38454 + AudioChannel, 0);
				break;
			case ACT_PLUGINSETUP:
				// note: this will block the remote!
				ConfigDialog(NULL);
				break;
			}
			break;
		case ACTF_F_KEYDOWN:
			if ((ActionCode & ACTF_KEY_NOSFW) == 0) ForceForeground(hVideo);
			KeyDown(ActionCode);
			break;
		case ACTF_F_WMCOMMAND_V:
			PostMessage(hVideoSkin, WM_COMMAND, MAKELONG((ActionCode & ACTF_WMCOMMAND_VALUE_MASK), 1), 0);
			break;
		case ACTF_F_WMCOMMAND_P:
			PostMessage(hPlayerSkin, WM_COMMAND, MAKELONG((ActionCode & ACTF_WMCOMMAND_VALUE_MASK), 1), 0);
			break;
		}
	}

	return TRUE;
}

int AreYouInFocus (void) 
{
	HWND hWnd;
	TCHAR s[256];
	if ((hWnd = GetForegroundWindow()) == NULL) return 0;
	if (GetClassName(hWnd, s, sizeof(s)/sizeof(s[0])) == 0) return 0;
	return (_tcscmp(s, _T("WinDVDClass")) == 0);
}


